@@ -1,10 +1,13 @@
+#ifndef _POSIX_C_SOURCE
+#define _POSIX_C_SOURCE 200809L
+#endif
+
 #if HAVE_CONFIG_H
 #include <config.h>
 #endif
 #include "data-pool.h"
 #include "maxminddb-compat-util.h"
 #include "maxminddb.h"
-#include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <inttypes.h>
@@ -925,7 +928,7 @@ static int find_address_in_search_tree(const MMDB_s *const mmdb,
                                        sa_family_t address_family,
                                        MMDB_lookup_result_s *result) {
     record_info_s record_info = record_info_for_database(mmdb);
-    if (0 == record_info.right_record_offset) {
+    if (record_info.right_record_offset == 0) {
         return MMDB_UNKNOWN_DATABASE_FORMAT_ERROR;
     }
 
@@ -989,10 +992,11 @@ static record_info_s record_info_for_database(const MMDB_s *const mmdb) {
         record_info.left_record_getter = &get_uint32;
         record_info.right_record_getter = &get_uint32;
         record_info.right_record_offset = 4;
-    } else {
-        assert(false);
     }
 
+    // Callers must check that right_record_offset is non-zero in case none of
+    // the above conditions matched.
+
     return record_info;
 }
 
@@ -1005,6 +1009,9 @@ static int find_ipv4_start_node(MMDB_s *const mmdb) {
     }
 
     record_info_s record_info = record_info_for_database(mmdb);
+    if (record_info.right_record_offset == 0) {
+        return MMDB_UNKNOWN_DATABASE_FORMAT_ERROR;
+    }
 
     const uint8_t *search_tree = mmdb->file_content;
     uint32_t node_value = 0;
@@ -1067,7 +1074,7 @@ int MMDB_read_node(const MMDB_s *const mmdb,
                    uint32_t node_number,
                    MMDB_search_node_s *const node) {
     record_info_s record_info = record_info_for_database(mmdb);
-    if (0 == record_info.right_record_offset) {
+    if (record_info.right_record_offset == 0) {
         return MMDB_UNKNOWN_DATABASE_FORMAT_ERROR;
     }
 
@@ -1808,12 +1815,16 @@ static void free_mmdb_struct(MMDB_s *const mmdb) {
     }
 
     if (NULL != mmdb->filename) {
+#if defined(__clang__)
 // This is a const char * that we need to free, which isn't valid. However it
 // would mean changing the public API to fix this.
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wcast-qual"
+#endif
         FREE_AND_SET_NULL(mmdb->filename);
+#if defined(__clang__)
 #pragma clang diagnostic pop
+#endif
     }
     if (NULL != mmdb->file_content) {
 #ifdef _WIN32
@@ -1822,22 +1833,30 @@ static void free_mmdb_struct(MMDB_s *const mmdb) {
          * to cleanup then. */
         WSACleanup();
 #else
+#if defined(__clang__)
 // This is a const char * that we need to free, which isn't valid. However it
 // would mean changing the public API to fix this.
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wcast-qual"
+#endif
         munmap((void *)mmdb->file_content, (size_t)mmdb->file_size);
+#if defined(__clang__)
 #pragma clang diagnostic pop
+#endif
 #endif
     }
 
     if (NULL != mmdb->metadata.database_type) {
+#if defined(__clang__)
 // This is a const char * that we need to free, which isn't valid. However it
 // would mean changing the public API to fix this.
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wcast-qual"
+#endif
         FREE_AND_SET_NULL(mmdb->metadata.database_type);
+#if defined(__clang__)
 #pragma clang diagnostic pop
+#endif
     }
 
     free_languages_metadata(mmdb);
@@ -1850,12 +1869,16 @@ static void free_languages_metadata(MMDB_s *mmdb) {
     }
 
     for (size_t i = 0; i < mmdb->metadata.languages.count; i++) {
+#if defined(__clang__)
 // This is a const char * that we need to free, which isn't valid. However it
 // would mean changing the public API to fix this.
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wcast-qual"
+#endif
         FREE_AND_SET_NULL(mmdb->metadata.languages.names[i]);
+#if defined(__clang__)
 #pragma clang diagnostic pop
+#endif
     }
     FREE_AND_SET_NULL(mmdb->metadata.languages.names);
 }
@@ -1868,24 +1891,32 @@ static void free_descriptions_metadata(MMDB_s *mmdb) {
     for (size_t i = 0; i < mmdb->metadata.description.count; i++) {
         if (NULL != mmdb->metadata.description.descriptions[i]) {
             if (NULL != mmdb->metadata.description.descriptions[i]->language) {
+#if defined(__clang__)
 // This is a const char * that we need to free, which isn't valid. However it
 // would mean changing the public API to fix this.
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wcast-qual"
+#endif
                 FREE_AND_SET_NULL(
                     mmdb->metadata.description.descriptions[i]->language);
+#if defined(__clang__)
 #pragma clang diagnostic pop
+#endif
             }
 
             if (NULL !=
                 mmdb->metadata.description.descriptions[i]->description) {
+#if defined(__clang__)
 // This is a const char * that we need to free, which isn't valid. However it
 // would mean changing the public API to fix this.
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wcast-qual"
+#endif
                 FREE_AND_SET_NULL(
                     mmdb->metadata.description.descriptions[i]->description);
+#if defined(__clang__)
 #pragma clang diagnostic pop
+#endif
             }
             FREE_AND_SET_NULL(mmdb->metadata.description.descriptions[i]);
         }
